from nicoclient.core.lookup import get_video_by_id
from nicoclient.core.search import search_utattemita_videos, search_daily_trending_utatttemita_videos
from nicoclient.html_page.playlist import get_playlist

name = "nicoclient"
